/*
    Copyright (C) 2005-2009  Michel de Boer <michel@twinklephone.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "hdr_referred_by.h"
#include "definitions.h"
#include "parse_ctrl.h"
#include "util.h"

t_hdr_referred_by::t_hdr_referred_by() : t_header("Referred-By", "b") {}

void t_hdr_referred_by::set_display(const string &d) {
	populated = true;
	display = d;
}

void t_hdr_referred_by::set_uri(const string &u) {
	populated = true;
	uri.set_url(u);
}

void t_hdr_referred_by::set_uri(const t_url &u) {
	populated = true;
	uri = u;
}

void t_hdr_referred_by::set_cid(const string &c) {
	populated = true;
	cid = c;
}

void t_hdr_referred_by::set_params(const list<t_parameter> &l) {
	populated = true;
	params = l;
}

void t_hdr_referred_by::add_param(const t_parameter &p) {
	populated = true;
	params.push_back(p);
}

string t_hdr_referred_by::encode_value(void) const {
	string s;

	if (!populated) return s;

	if (display.size() > 0) {
		s += '"';
		s += escape(display, '"');
		s += '"';
		s += ' ';
	}

	s += '<';
	s += uri.encode();
	s += '>';

	if (cid.size() > 0) {
		s += ";cid=";
		s += cid;
	}

	s += param_list2str(params);

	return s;
}
